        page    ,132

;*****************************************************************************
;
;  SMART HOSTESS Asynchronous Control Program Skeleton
;
;  Copyright (C) 1987 Control Systems, Inc.  All rights reserved.
;
;  This is a stripped down version of Control Systems' control program that
;  runs on the SMART HOSTESS board.  This is only to be used as a guide in
;  writing other control programs, and will probably not assemble or run
;  as is.
;
;  The information and design disclosed herein were originated by and are
;  the property of Control Systems, Inc.  This software may only be used
;  on or in conjunction with the Control Systems, Inc. SMART HOSTESS
;  board.
;
;*****************************************************************************

        .xlist
        include \include\xx186.equ
        include \include\iapx186.equ
        include \include\scc.equ
        include \include\dpram.equ
        .list

pgm     segment public  'code'
        assume  cs:pgm, ds:pgm, ss:pgm, es:nothing

        db      firmware_data_len dup (?)	;128 bytes long

        public  pgm_start
pgm_start       equ     $

        jmp     near ptr relocate       ;relocate program to local memory

;----------------------------------------------------------------------------
;  control is passed to 'main' from the relocator

main    proc
        mov     ax,cs           	;establish data segment
        mov     ds,ax

        cli                     	;disable interrupts
        mov     ss,ax           	;establish a program stack
        mov     sp,offset pgm_start

	xor	ax,ax
	mov	es,ax			;es = segment 0
	mov	bx,INT1_type*4		;vector table address of sys intr
	mov	ax,offset system_isr	;system interrupt service routine
	mov	es:[bx],ax		;move offset into vector table
	mov	ax,cs
	mov	es:[bx+2],ax		;move segment into vector table
        sti                     	;re-enable interrupts

	mov	ax,dpram_segment
	mov	es,ax			;es = dual-ported mem segment
	xor	es:interact_flag,0ffffh	;indicate relocation complete

main_02:
        mov     ax,base_vector  	;ax = base vector type
        call    vector_init     	;initialize vector table

        mov     ax,dpram_segment
        mov     es,ax			;es = dual ported RAM
        mov     si,offset line0 	;es:si = address of 1st line table
main_04:
        call    async_init      	;initialize the line
        add     si,line_entry_len	;es:si = address of next line table
        cmp     si,offset line7+line_entry_len  ;last table ?
        jnz     main_04                         ;no ... continue

	;everything is now set up and initialized
	;go into an infinite loop, polling lines to see if they need
	;servicing

main    endp

;----------------------------------------------------------------------------
;  name: system_isr
;  function:  process interrupt from system processor

system_isr	proc
	sti
	push	ax				;save registers
	push	bx
	push	cx
	push	dx
	push	si
	push	di
	push	ds
	push	es

	mov	ax,cs				;establish data segment
	mov	ds,ax

	mov	dx,INT_reg		;clear system processor interrupt
	out	dx,al

	;process the interrupt request...

	mov	dx,EOI_reg		;allow further interrupts
	mov	ax,8000h
	out	dx,ax

	pop	es				;restore registers
	pop	ds
	pop	di
	pop	si
	pop	dx
	pop	cx
	pop	bx
	pop	ax
	iret
system_isr	endp

;----------------------------------------------------------------------------
;  name: async_init
;
;  function: startup asynchronous communications channel
;
;  For interrupt driven, asynchronous communications, the following
;  initialization sequence is used:
;
;	WR4: specify clock mode, number of stop bits and parity
;	WR2: specify base interrupt vector type
;	WR3: specify number of bits/character for receive
;	WR5: specify number of bits/character for transmit
;	WR9: specify interrupt control
;	WR11: specify receive and transmit clock source
;	WR12: specify lower byte of Baud Rate Time Constant
;	WR13: specify upper byte of Baud Rate Time Constant
;	WR14: specify Baud Rate Generator source and enable it
;	WR3: enable receive operation
;	WR5: enable transmit operation
;	WR15: specify external/status interrupts
;	WR0: reset external status
;	WR0: reset external status
;	WR1: enable receive, transmit and external status interrupts
;	WR9: enable master interrupt enable
;
;  enter: ES:SI=> line table entry
;  exit: no parameters returned
;
;	NOTE: io_base and all the WR#_images must be set to valid values
;	before they are sent out to the board.  Values may be set either
;	as default values, or by the initial device driver code.
;
;	A detailed description of the write registers can be found in th
;	Serial Communications Controller Technical Manual
;
;	Here is an example of one line table set to default values:
;
;	line 0:
;	dw	0004			;io_base
;	db	80h			;WR2 - interrupt vector
;	db	0c0h			;WR3 - Rx 8 bits/char
;	db	04h			;WR4 - 1 stop bit, no parity
;	db	60h			;WR5 - Tx 8 bits/char
;	db	0eh			;WR12 - 9600 bits/sec (lsb)
;	db	00h			;WR13 - 9600 bits/sec (msb)
;	db	00h			;WR15 - external/status intr control

async_init      proc
        mov     dx,es:[si].io_base      ; get base I/O address

        mov     al,WR4
        out     dx,al
        mov     al,es:[si].WR4_image    ; stop bits and parity
        or      al,x16_clock            ; add clock mode
        out     dx,al

        mov     al,WR2
        out     dx,al
        mov     al,es:[si].WR2_image    ; base interrupt vector type 
        out     dx,al

        mov     al,WR3
        out     dx,al
        mov     al,es:[si].WR3_image    ; Rx character size
        out     dx,al

        mov     al,WR5
        out     dx,al
        mov     al,es:[si].WR5_image    ; Tx character size, modem/break
        out     dx,al

        mov     al,WR9
        out     dx,al
        mov     al,status_lo+VIS        ; interrupt control
        out     dx,al

        mov     al,WR11
        out     dx,al
        mov     al,Rx_clk_eq_BRG+Tx_clk_eq_BRG  ; clock sources
        out     dx,al

        mov     al,WR12
        out     dx,al
        mov     al,es:[si].WR12_image   ; lower byte of BRGTC
        out     dx,al

        mov     al,WR13
        out     dx,al
        mov     al,es:[si].WR13_image   ; upper byte of BRGTC
        out     dx,al

        mov     al,WR14
        out     dx,al
        mov     al,BRG_eq_sys_clk+BRG_enable    ; BRG source
        out     dx,al

        mov     al,WR3
        out     dx,al
        mov     al,es:[si].WR3_image    ; Rx character size
        or      al,Rx_enable            ; enable receive
        out     dx,al

        mov     al,WR5
        out     dx,al
        mov     al,es:[si].WR5_image    ; Tx character size
        or      al,Tx_enable            ; enable transmit
        out     dx,al

        mov     al,WR15
        out     dx,al
        mov     al,es:[si].WR15_image   ; external/status interrupt control
        out     dx,al

        mov     al,reset_ext            ; reset external status interrupts
        out     dx,al
        out     dx,al

        mov     al,WR1                  ; interrupt enables
        out     dx,al
        mov     al,ext_int_enable+Tx_int_enable+parity_special+Rx_int_enable
        out     dx,al

        mov     al,WR9
        out     dx,al
        mov     al,MIE+status_lo+VIS    ; master interrupt enable
        out     dx,al
        ret
async_init      endp

;----------------------------------------------------------------------------
;  Interrupt Service Routines - i.e. when SCC interrupts the comm processor

;----------------------------------------------------------------------------
;  LINE 0

line0_TBE       proc			;Transmit Buffer Empty
        push    si			;save register
        mov     si,offset line0		;and setup line table offset
        jmp     TBE_isr			;do common processing ...
line0_TBE       endp

line0_ESC       proc			;External/Status Change
        push    si			;save register
        mov     si,offset line0 	;and setup line table offset
        jmp     ESC_isr			;do common processing ...
line0_ESC       endp

line0_RCA       proc			;Receive Character Available
        push    si			;save register
        mov     si,offset line0		;and setup interrupt type
        jmp     RCA_isr			;do common processing ...
line0_RCA       endp

line0_SRC       proc			;Special Receive Condition
        push    si			;save register
        mov     si,offset line0		;and setup interrupt type
        jmp     SRC_isr			;do common processing ...
line0_SRC       endp

;----------------------------------------------------------------------------
;  LINE1

line1_TBE       proc			;Transmit Buffer Empty
        push    si			;save register
        mov     si,offset line1		;and setup line table offset
        jmp     TBE_isr			;do common processing ...
line1_TBE       endp

line1_ESC       proc			;External/Status Change
        push    si			;save register
        mov     si,offset line1		;and setup line table offset
        jmp     ESC_isr			;do common processing ...
line1_ESC       endp

line1_RCA       proc			;Receive Character Available
        push    si			;save register
        mov     si,offset line1		;and setup interrupt type
        jmp     RCA_isr			;do common processing ...
line1_RCA       endp

line1_SRC       proc			;Special Receive Condition
        push    si			;save register
        mov     si,offset line1		;and setup interrupt type
        jmp     SRC_isr			;do common processing ...
line1_SRC       endp

;----------------------------------------------------------------------------
;  LINE 2

line2_TBE       proc			;Transmit Buffer Empty
        push    si			;save register
        mov     si,offset line2		;and setup line table offset
        jmp     TBE_isr			;do common processing ...
line2_TBE       endp

line2_ESC       proc			;External/Status Change
        push    si			;save register
        mov     si,offset line2		;and setup line table offset
        jmp     ESC_isr			;do common processing ...
line2_ESC       endp

line2_RCA       proc			;Receive Character Available
        push    si			;save register
        mov     si,offset line2		;and setup interrupt type
        jmp     RCA_isr			;do common processing ...
line2_RCA       endp

line2_SRC       proc			;Special Receive Condition
        push    si			;save register
        mov     si,offset line2		;and setup interrupt type
        jmp     SRC_isr			;do common processing ...
line2_SRC       endp

;----------------------------------------------------------------------------
;  LINE 3

line3_TBE       proc			;Transmit Buffer Empty
        push    si			;save register
        mov     si,offset line3		;and setup line table offset
        jmp     TBE_isr			;do common processing ...
line3_TBE       endp

line3_ESC       proc			;External/Status Change
        push    si			;save register
        mov     si,offset line3		;and setup line table offset
        jmp     ESC_isr			;do common processing ...
line3_ESC       endp

line3_RCA       proc			;Receive Character Available
        push    si			;save register
        mov     si,offset line3		;and setup interrupt type
        jmp     RCA_isr			;do common processing ...
line3_RCA       endp

line3_SRC       proc			;Special Receive Condition
        push    si			;save register
        mov     si,offset line3		;and setup interrupt type
        jmp     SRC_isr			;do common processing ...
line3_SRC       endp

;----------------------------------------------------------------------------
;  LINE 4

line4_TBE       proc			;Transmit Buffer Empty
        push    si			;save register
        mov     si,offset line4		;and setup line table offset
        jmp     TBE_isr			;do common processing ...
line4_TBE       endp

line4_ESC       proc			;External/Status Change
        push    si			;save register
        mov     si,offset line4		;and setup line table offset
        jmp     ESC_isr			;do common processing ...
line4_ESC       endp

line4_RCA       proc			;Receive Character Available
        push    si			;save register
        mov     si,offset line4		;and setup interrupt type
        jmp     RCA_isr			;do common processing ...
line4_RCA       endp

line4_SRC       proc			;Special Receive Condition
        push    si			;save register
        mov     si,offset line4		;and setup interrupt type
        jmp     SRC_isr			;do common processing ...
line4_SRC       endp

;----------------------------------------------------------------------------
;  LINE 5

line5_TBE       proc			;Transmit Buffer Empty
        push    si			;save register
        mov     si,offset line5		;and setup line table offset
        jmp     TBE_isr			;do common processing ...
line5_TBE       endp

line5_ESC       proc			;External/Status Change
        push    si			;save register
        mov     si,offset line5		;and setup line table offset
        jmp     ESC_isr			;do common processing ...
line5_ESC       endp

line5_RCA       proc			;Receive Character Available
        push    si			;save register
        mov     si,offset line5		;and setup interrupt type
        jmp     RCA_isr			;do common processing ...
line5_RCA       endp

line5_SRC       proc			;Special Receive Condition
        push    si			;save register
        mov     si,offset line5		;and setup interrupt type
        jmp     SRC_isr			;do common processing ...
line5_SRC       endp

;----------------------------------------------------------------------------
;  LINE 6

line6_TBE       proc			;Transmit Buffer Empty
        push    si			;save register
        mov     si,offset line6		;and setup line table offset
        jmp     TBE_isr			;do common processing ...
line6_TBE       endp

line6_ESC       proc			;External/Status Change
        push    si			;save register
        mov     si,offset line6		;and setup line table offset
        jmp     ESC_isr			;do common processing ...
line6_ESC       endp

line6_RCA       proc			;Receive Character Available
        push    si			;save register
        mov     si,offset line6		;and setup interrupt type
        jmp     RCA_isr			;do common processing ...
line6_RCA       endp

line6_SRC       proc			;Special Receive Condition
        push    si			;save register
        mov     si,offset line6		;and setup interrupt type
        jmp     SRC_isr			;do common processing ...
line6_SRC       endp

;----------------------------------------------------------------------------
;  LINE 7

line7_TBE       proc			;Transmit Buffer Empty
        push    si			;save register
        mov     si,offset line7		;and setup line table offset
        jmp     TBE_isr			;do common processing ...
line7_TBE       endp

line7_ESC       proc			;External/Status Change
        push    si			;save register
        mov     si,offset line7		;and setup line table offset
        jmp     ESC_isr			;do common processing ...
line7_ESC       endp

line7_RCA       proc			;Receive Character Available
        push    si			;save register
        mov     si,offset line7		;and setup interrupt type
        jmp     RCA_isr			;do common processing ...
line7_RCA       endp

line7_SRC       proc			;Special Receive Condition
        push    si			;save register
        mov     si,offset line7		;and setup interrupt type
        jmp     SRC_isr			;do common processing ...
line7_SRC       endp

;----------------------------------------------------------------------------
;  Common Interrupt Service Routines

;----------------------------------------------------------------------------
;  Transmit Buffer Empty Interrupt Service Routine

TBE_isr proc
        push    ax			;save registers
        push    bx
        push    cx
        push    dx
        push    di
        push    ds
        push    es

	;code to process the interrupt...

        jmp     isr_ret			;common exit
TBE_isr endp

;----------------------------------------------------------------------------
;  External Status Change Interrupt Service Routine

ESC_isr proc
        push    ax			;save registers
        push    bx
        push    cx
        push    dx
        push    di
        push    ds
        push    es

	;code to process the interrupt...

        jmp     isr_ret			;common exit
ESC_isr endp

;----------------------------------------------------------------------------
;  Receive Character Available Interrupt Service Routine

RCA_isr proc
        push    ax			;save registers
        push    bx
        push    cx
        push    dx
        push    di
        push    ds
        push    es

	;code to process the interrupt...

        jmp     isr_ret			;common exit
RCA_isr endp

;----------------------------------------------------------------------------
;  Special Receive Condition Interrupt Service Routine

SRC_isr proc
        push    ax			;save registers
        push    bx
        push    cx
        push    dx
        push    di
        push    ds
        push    es

	;code to process the interrupt...

SRC_isr endp

;----------------------------------------------------------------------------
;  name: isr_ret
;  function: common Interrupt Service Routine exit processing
;
;  enter: ES:SI=> line table entry,
;         interrupted AX, DX, SI and ES on the stack
;  exit: to interrupted routine

isr_ret proc
        mov     dx,es:[si].io_base		;get base I/O address
        mov     al,reset_ius    		;indicate end of interrupt
        out     dx,al				;to SCC

        mov     dx,EOI_reg			;indicate end of interrupt
        mov     ax,8000h			;to interrupt controller
        out     dx,ax

        pop     es				;recover registers
        pop     ds
        pop     di
        pop     dx
        pop     cx
        pop     bx
        pop     ax
        pop     si
        iret
isr_ret endp

;----------------------------------------------------------------------------
;  name: vector_init
;  function: initialize interrupt vectors
;  enter: AX = base vector type
;  exit: no parameters returned
;  registers AX, BX, CX, SI, DI and ES altered

vector_init     proc
        shl     ax,1            ;interrupt vector address = type * 4
        shl     ax,1
        mov     di,ax		;di = address of first interrupt vector
        xor     ax,ax		;segment = 0
        mov     es,ax           ;es = 0 segment

        mov     si,offset vector_tbl    ;si = address of vector table
        mov     cx,[si]		;cx = byte length of table
        add     si,2		;bump up pointer to next entry in table
        shr     cx,1            ;cx = word length of table
        mov     bx,cs           ;setup segment address
vector_init_10:
        movsw                   ;move routine offset into vector table
        mov     ax,bx
        stosw			;move routine segment into vector table
        add     di,4            ;leave next 4 bytes in vector table empty
        loop    vector_init_10  ;continue for entire table
        ret
vector_init     endp

;----------------------------------------------------------------------------
;  Interrupt Vector Table holds the addresses of the individual routines
;	that will be called at an interrupt

vector_tbl      equ     $
        dw      vector_tbl_end-$-2      ; table length

        dw      line1_TBE
        dw      line1_ESC
        dw      line1_RCA
        dw      line1_SRC

        dw      line0_TBE
        dw      line0_ESC
        dw      line0_RCA
        dw      line0_SRC

        dw      line3_TBE
        dw      line3_ESC
        dw      line3_RCA
        dw      line3_SRC

        dw      line2_TBE
        dw      line2_ESC
        dw      line2_RCA
        dw      line2_SRC

        dw      line5_TBE
        dw      line5_ESC
        dw      line5_RCA
        dw      line5_SRC

        dw      line4_TBE
        dw      line4_ESC
        dw      line4_RCA
        dw      line4_SRC

        dw      line7_TBE
        dw      line7_ESC
        dw      line7_RCA
        dw      line7_SRC

        dw      line6_TBE
        dw      line6_ESC
        dw      line6_RCA
        dw      line6_SRC
vector_tbl_end  equ     $

        dw      64 dup ('@@')
temp_stack      equ     $

;----------------------------------------------------------------------------
;  relocate moves the control program from dual ported memory to local
;	comm processor memory.

relocate        proc    far
	mov     ax,cs
	mov     ds,ax			;establish user program data segment
	cli				;disable interrupts
	mov     ss,ax			;establish temporary stack
	mov     sp,offset temp_stack
	sti				;re-enable interrupts

	int     21h	;firmware interrupt: RAM query for open space
	push    ax			;and save it
	mov     es,ax
	mov     di,offset pgm_start	;ES:DI=> destination
	mov     si,offset pgm_start	;DS:SI=> source
	mov     cx,offset pgm_end
	sub     cx,si			;calculate program length
	rep     movsb			;relocate program
	mov     ax,offset main		;setup 'return' address
	push    ax
	ret				;exit to relocated program ...
relocate        endp

;----------------------------------------------------------------------------

	public  pgm_end
pgm_end equ     $

pgm     ends
	end
